@@ -1,14 +1,14 @@
 from problog.extern import problog_export, problog_export_nondet, problog_export_raw
 
 
-@problog_export_raw('+term')
+@problog_export_raw("+term")
 def assertz(term, target=None, **kwargs):
     problog_export.database += term
     target._cache.reset()  # reset tabling cache
     return [(term,)]
 
 
-@problog_export_raw('+term')
+@problog_export_raw("+term")
 def retract(term, target=None, **kwargs):
     db = problog_export.database
     nodekey = db.find(term)
@@ -23,7 +23,7 @@ def retract(term, target=None, **kwargs):
         return []
 
 
-@problog_export_raw('+term')
+@problog_export_raw("+term")
 def retractall(term, target=None, **kwargs):
     db = problog_export.database
     nodekey = db.find(term)
@@ -31,4 +31,4 @@ def retractall(term, target=None, **kwargs):
     to_erase = node.children.find(term.args)
     node.children.erase(to_erase)
     target._cache.reset()  # reset tabling cache
-    return [(term,)]
\ No newline at end of file
+    return [(term,)]
