@@ -28,7 +28,7 @@ from collections import Counter, defaultdict, OrderedDict
 import logging
 
 
-logger = logging.getLogger('be.kuleuven.cs.dtai.problog.cpd')
+logger = logging.getLogger("be.kuleuven.cs.dtai.problog.cpd")
 
 
 class PGM(object):
@@ -46,7 +46,7 @@ class PGM(object):
         if name:
             self.name = name
         else:
-            self.name = 'PGM {}'.format(PGM.__count)
+            self.name = "PGM {}".format(PGM.__count)
             PGM.__count += 1
         self.comments = []
         self.directed = directed
@@ -60,16 +60,16 @@ class PGM(object):
                 self.add_factor(factor)
 
     def copy(self, **kwargs):
-        variables = kwargs.get('variables', None)
+        variables = kwargs.get("variables", None)
         if variables is None:
             variables = list(self.vars.values())
-        factors = kwargs.get('factors', None)
+        factors = kwargs.get("factors", None)
         if factors is None:
             factors = list(self.factors.values())
         return PGM(
-            directed=kwargs.get('directed', self.directed),
+            directed=kwargs.get("directed", self.directed),
             factors=factors,
-            variables=variables
+            variables=variables,
         )
 
     def split_topological(self, split_vars):
@@ -108,9 +108,11 @@ class PGM(object):
                 queue.extend(self.factors[qvar].parents)
             else:
                 first_roots.add(qvar)
-        delay_roots= root - first_roots
+        delay_roots = root - first_roots
 
-        factor_strata = self.factors_topological(keep_strata=True, delay_roots=[delay_roots])
+        factor_strata = self.factors_topological(
+            keep_strata=True, delay_roots=[delay_roots]
+        )
         # for fs_i, fs in enumerate(factor_strata):
         #     print('{}: {}'.format(fs_i, ', '.join([f.rv for f in fs])))
         strata = dict()
@@ -120,14 +122,21 @@ class PGM(object):
         max_strata = max((strata[svar] for svar in split_vars))
 
         def rootvar(var):
-            return var+"_root"
+            return var + "_root"
+
         pgm = self.copy(factors=[])
         for factor in self.factors.values():
-            if strata[factor.rv] <= max_strata and any((strata[cvar] > max_strata for cvar in children[factor.rv])):
+            if strata[factor.rv] <= max_strata and any(
+                (strata[cvar] > max_strata for cvar in children[factor.rv])
+            ):
                 nb_values = len(self.vars[factor.rv].values)
                 pgm.add_var(self.vars[factor.rv].copy(name=rootvar(factor.rv)))
-                pgm.add_factor(Factor(pgm, rootvar(factor.rv), [], [1.0 / nb_values] * nb_values))
-            if strata[factor.rv] > max_strata and any((strata[pvar] <= max_strata for pvar in factor.parents)):
+                pgm.add_factor(
+                    Factor(pgm, rootvar(factor.rv), [], [1.0 / nb_values] * nb_values)
+                )
+            if strata[factor.rv] > max_strata and any(
+                (strata[pvar] <= max_strata for pvar in factor.parents)
+            ):
                 parents = []
                 for pvar in factor.parents:
                     if strata[pvar] <= max_strata:
@@ -148,14 +157,18 @@ class PGM(object):
 
         :param variables: Set of variable names.
         """
+
         def splitvar(var):
-            return var+"_sink"
+            return var + "_sink"
+
         new_vars = [self.vars[var].copy(name=splitvar(var)) for var in variables]
         pgm = self.copy(factors=[], variables=list(self.vars.values()) + new_vars)
         for factor in self.factors.values():
             if factor.rv in variables:
                 nb_values = len(self.vars[factor.rv].values)
-                pgm.add_factor(Factor(pgm, factor.rv, [], [1.0/nb_values]*nb_values))
+                pgm.add_factor(
+                    Factor(pgm, factor.rv, [], [1.0 / nb_values] * nb_values)
+                )
                 pgm.add_factor(factor.copy(pgm=pgm, rv=splitvar(factor.rv)))
             else:
                 pgm.add_factor(factor.copy(pgm=pgm))
@@ -185,16 +198,16 @@ class PGM(object):
                 nb_parts += 1
                 cur_part = nb_parts
             parts[var] = cur_part
-            logger.debug('PGM[{}] = {}'.format(var, cur_part))
+            logger.debug("PGM[{}] = {}".format(var, cur_part))
             for cvar in children[var]:
-                logger.debug('{} -> {}'.format(var, cvar))
+                logger.debug("{} -> {}".format(var, cvar))
                 queue.append((cvar, cur_part))
             for pvar in self.factors[var].parents:
-                logger.debug('{} -> {}'.format(pvar, var))
+                logger.debug("{} -> {}".format(pvar, var))
                 queue.append((pvar, cur_part))
         nb_parts += 1
         pgms = [self.copy(factors=[], variables=[]) for i in range(nb_parts)]
-        logger.debug('Creating {} PGMs'.format(len(pgms)))
+        logger.debug("Creating {} PGMs".format(len(pgms)))
         for var, part in parts.items():
             factor = self.factors[var]
             if factor.rv:
@@ -215,7 +228,7 @@ class PGM(object):
             name = factor.rv
         else:
             name = factor.name
-        assert (name is not None)
+        assert name is not None
         factor.pgm = self
         if name in self.factors:
             self.factors[name] += factor
@@ -259,7 +272,7 @@ class PGM(object):
                 try:
                     factor = self.factors[rv]
                 except KeyError as exc:
-                    print('Error: random variable has no CPD associated:')
+                    print("Error: random variable has no CPD associated:")
                     print(exc)
                     sys.exit(1)
                 for parent_rv in factor.parents:
@@ -297,17 +310,19 @@ class PGM(object):
         """
         assert self.directed
         cpds = [cpd.to_factor() for cpd in self.factors_topological()]
-        lines = ["%% Hugin Net: {}\n".format(self.name),
-                 "%% Created on {}\n".format(datetime.now())]
+        lines = [
+            "%% Hugin Net: {}\n".format(self.name),
+            "%% Created on {}\n".format(datetime.now()),
+        ]
         if include_layout:
-            lines += ["net {",
-                      "  node_size = (50,50);",
-                      "}\n"]
+            lines += ["net {", "  node_size = (50,50);", "}\n"]
         lines += ["%% Nodes\n"]
         lines += [cpd.to_huginnet_node(include_layout=include_layout) for cpd in cpds]
         lines += ["%% Potentials\n"]
-        lines += [cpd.to_huginnet_potential(include_layout=include_layout) for cpd in cpds]
-        return '\n'.join(lines)
+        lines += [
+            cpd.to_huginnet_potential(include_layout=include_layout) for cpd in cpds
+        ]
+        return "\n".join(lines)
 
     def to_xdsl(self):
         """Export PGM to the XDSL format defined by SMILE.
@@ -316,18 +331,22 @@ class PGM(object):
         """
         assert self.directed
         cpds = [cpd.to_factor() for cpd in self.factors_topological()]
-        lines = ['<?xml version="1.0" encoding="ISO-8859-1" ?>',
-                 '<smile version="1.0" id="Aa" numsamples="1000">',
-                 '  <nodes>']
+        lines = [
+            '<?xml version="1.0" encoding="ISO-8859-1" ?>',
+            '<smile version="1.0" id="Aa" numsamples="1000">',
+            "  <nodes>",
+        ]
         lines += [cpd.to_xdsl_cpt() for cpd in cpds]
-        lines += ['  </nodes>',
-                  '  <extensions>',
-                  '    <genie version="1.0" app="ProbLog" name="{}" faultnameformat="nodestate">'.format(self.name)]
+        lines += [
+            "  </nodes>",
+            "  <extensions>",
+            '    <genie version="1.0" app="ProbLog" name="{}" faultnameformat="nodestate">'.format(
+                self.name
+            ),
+        ]
         lines += [cpd.to_xdsl_node() for cpd in cpds]
-        lines += ['    </genie>',
-                  '  </extensions>',
-                  '</smile>']
-        return '\n'.join(lines)
+        lines += ["    </genie>", "  </extensions>", "</smile>"]
+        return "\n".join(lines)
 
     def to_xmlbif(self):
         """Export PGM to the XMLBIF format defined in
@@ -335,14 +354,15 @@ class PGM(object):
         """
         assert self.directed
         cpds = [cpd.to_factor() for cpd in self.factors_topological()]
-        lines = ['<?xml version="1.0" encoding="US-ASCII"?>',
-                 '<BIF VERSION="0.3">',
-                 '<NETWORK>']
+        lines = [
+            '<?xml version="1.0" encoding="US-ASCII"?>',
+            '<BIF VERSION="0.3">',
+            "<NETWORK>",
+        ]
         lines += [cpd.to_xmlbif_node() for cpd in cpds]
         lines += [cpd.to_xmlbif_cpt() for cpd in cpds]
-        lines += ['</NETWORK>',
-                  '</BIF>']
-        return '\n'.join(lines)
+        lines += ["</NETWORK>", "</BIF>"]
+        return "\n".join(lines)
 
     def to_uai08(self):
         """Export PGM to the format used in the UAI 2008 competition.
@@ -355,16 +375,19 @@ class PGM(object):
         number_variables = str(len(cpds))
         domain_sizes = [str(len(self.vars[cpd.rv].values)) for cpd in cpds]
         number_functions = str(len(cpds))
-        lines = ['BAYES',
-                 number_variables,
-                 ' '.join(domain_sizes),
-                 number_functions]
+        lines = ["BAYES", number_variables, " ".join(domain_sizes), number_functions]
         lines += [cpd.to_uai08_preamble(cpds) for cpd in cpds]
-        lines += ['']
+        lines += [""]
         lines += [cpd.to_uai08_function() for cpd in cpds]
-        return '\n'.join(lines)
+        return "\n".join(lines)
 
-    def to_problog(self, drop_zero=False, use_neglit=False, value_as_term=True, ad_is_function=False):
+    def to_problog(
+        self,
+        drop_zero=False,
+        use_neglit=False,
+        value_as_term=True,
+        ad_is_function=False,
+    ):
         """Export PGM to ProbLog.
         :param ad_is_function: Experimental
         :param value_as_term: Include the variable's value as the last term instead of as part of the predicate name
@@ -372,24 +395,38 @@ class PGM(object):
         :param drop_zero: Do not include head literals with probability zero
         """
         factors = [factor.to_factor() for factor in self.factors_topological()]
-        lines = ["%% ProbLog program: {}".format(self.name),
-                 "%% Created on {}".format(datetime.now())]
+        lines = [
+            "%% ProbLog program: {}".format(self.name),
+            "%% Created on {}".format(datetime.now()),
+        ]
         if len(self.comments) > 0:
             lines += ["%% {}".format(comment) for comment in self.comments]
         lines += [""]
         if self.directed:
-            lines += [factor.to_problog(self, drop_zero=drop_zero,
-                                        use_neglit=use_neglit,
-                                        value_as_term=value_as_term,
-                                        ad_is_function=ad_is_function) for factor in factors]
+            lines += [
+                factor.to_problog(
+                    self,
+                    drop_zero=drop_zero,
+                    use_neglit=use_neglit,
+                    value_as_term=value_as_term,
+                    ad_is_function=ad_is_function,
+                )
+                for factor in factors
+            ]
         else:
-            lines += [factor.to_problog_undirected(self, drop_zero=drop_zero,
-                                                   use_neglit=use_neglit,
-                                                   value_as_term=value_as_term,
-                                                   ad_is_function=ad_is_function) for factor in factors]
+            lines += [
+                factor.to_problog_undirected(
+                    self,
+                    drop_zero=drop_zero,
+                    use_neglit=use_neglit,
+                    value_as_term=value_as_term,
+                    ad_is_function=ad_is_function,
+                )
+                for factor in factors
+            ]
             # if ad_is_function:
             # lines += ["evidence(false_constraints,false)."]
-        return '\n'.join(lines)
+        return "\n".join(lines)
 
     def to_graphviz(self):
         """Export PGM to Graphviz dot format.
@@ -397,40 +434,51 @@ class PGM(object):
         """
         assert self.directed
         lines = [
-            '// Graphiz DOT: {}'.format(self.name),
+            "// Graphiz DOT: {}".format(self.name),
             "// Created on {}\n".format(datetime.now()),
-            'digraph pgm {'
+            "digraph pgm {",
         ]
         for cpd in self.factors.values():
             lines.append('  {} [label="{}"];'.format(self.vars[cpd.rv].clean(), cpd.rv))
             for p in cpd.parents:
-                lines.append('  {} -> {}'.format(self.vars[cpd.rv].clean(p), self.vars[cpd.rv].clean()))
-        lines += ['}']
-        return '\n'.join(lines)
+                lines.append(
+                    "  {} -> {}".format(
+                        self.vars[cpd.rv].clean(p), self.vars[cpd.rv].clean()
+                    )
+                )
+        lines += ["}"]
+        return "\n".join(lines)
 
     def __str__(self):
-        return '\n'.join([str(factor) for factor in self.factors.values()])
+        return "\n".join([str(factor) for factor in self.factors.values()])
 
 
-re_tounderscore = re.compile(r'[\(\),\]\[ ]')
+re_tounderscore = re.compile(r"[\(\),\]\[ ]")
 re_toremove = re.compile(r"""[^a-zA-Z0-9_]""")
 
 boolean_values = [
-    ['0', '1'],
+    ["0", "1"],
     [0, 1],
-    ['f', 't'],
-    ['false', 'true'],
-    ['no', 'yes'],
-    ['n', 'y'],
-    ['neg', 'pos'],
-    ['nay', 'aye'],
-    [False, True]
+    ["f", "t"],
+    ["false", "true"],
+    ["no", "yes"],
+    ["n", "y"],
+    ["neg", "pos"],
+    ["nay", "aye"],
+    [False, True],
 ]
 
 
 class Variable(object):
-    def __init__(self, name, values, detect_boolean=True, force_boolean=False, boolean_true=None,
-                 latent=False):
+    def __init__(
+        self,
+        name,
+        values,
+        detect_boolean=True,
+        force_boolean=False,
+        boolean_true=None,
+        latent=False,
+    ):
         """Conditional Probability Distribution."""
         self.name = name
         self.values = values
@@ -438,14 +486,20 @@ class Variable(object):
         self.boolean_true = boolean_true  # Value that represents true
         if (force_boolean or detect_boolean) and len(self.values) == 2:
             for values in boolean_values:
-                if (values[0] == self.values[0] and values[1] == self.values[1]) or \
-                   (type(self.values[0]) == str and type(self.values[1]) == str and
-                   values[0] == self.values[0].lower() and values[1] == self.values[1].lower()):
+                if (values[0] == self.values[0] and values[1] == self.values[1]) or (
+                    type(self.values[0]) == str
+                    and type(self.values[1]) == str
+                    and values[0] == self.values[0].lower()
+                    and values[1] == self.values[1].lower()
+                ):
                     self.boolean_true = 1
                     break
-                elif (values[1] == self.values[0] and values[0] == self.values[1]) or \
-                    (type(self.values[0]) == str and type(self.values[1]) == str and
-                        values[1] == self.values[0].lower() and values[0] == self.values[1].lower()):
+                elif (values[1] == self.values[0] and values[0] == self.values[1]) or (
+                    type(self.values[0]) == str
+                    and type(self.values[1]) == str
+                    and values[1] == self.values[0].lower()
+                    and values[0] == self.values[1].lower()
+                ):
                     self.boolean_true = 0
                     break
             if force_boolean and self.boolean_true is None:
@@ -453,12 +507,12 @@ class Variable(object):
 
     def copy(self, **kwargs):
         return Variable(
-            name=kwargs.get('name', self.name),
-            values=kwargs.get('values', self.values),
-            latent=kwargs.get('latent', self.latent),
+            name=kwargs.get("name", self.name),
+            values=kwargs.get("values", self.values),
+            latent=kwargs.get("latent", self.latent),
             detect_boolean=False,
             force_boolean=False,
-            boolean_true=kwargs.get('boolean_true', self.boolean_true),
+            boolean_true=kwargs.get("boolean_true", self.boolean_true),
         )
 
     def clean(self, value=None):
@@ -466,8 +520,8 @@ class Variable(object):
             rv = self.name
         else:
             rv = value
-        rv = re_tounderscore.sub('_', rv)
-        rv = re_toremove.sub('', rv)
+        rv = re_tounderscore.sub("_", rv)
+        rv = re_toremove.sub("", rv)
         if not rv[0].islower():
             rv = rv[0].lower() + rv[1:]
             if not rv[0].islower():
@@ -477,39 +531,55 @@ class Variable(object):
     def to_problog_value(self, value, value_as_term=True):
         if self.boolean_true is None:
             if type(value) is frozenset and len(value) == 1:
-                value, = value
+                (value,) = value
             if type(value) is frozenset:
                 # It is a disjunction of possible values
                 if len(value) == len(self.values) - 1:
                     # This is the negation of one value
-                    new_value, = frozenset(self.values) - value
+                    (new_value,) = frozenset(self.values) - value
                     if value_as_term:
-                        return '\+' + self.clean() + '("' + str(new_value) + '")'
+                        return "\+" + self.clean() + '("' + str(new_value) + '")'
                     else:
-                        return '\+' + self.clean() + '_' + self.clean(str(new_value))
+                        return "\+" + self.clean() + "_" + self.clean(str(new_value))
                 else:
                     if value_as_term:
-                        return '(' + '; '.join(
-                            [self.clean() + '("' + str(new_value) + '")' for new_value in value]) + ')'
+                        return (
+                            "("
+                            + "; ".join(
+                                [
+                                    self.clean() + '("' + str(new_value) + '")'
+                                    for new_value in value
+                                ]
+                            )
+                            + ")"
+                        )
                     else:
-                        return '(' + \
-                               '; '.join([self.clean() + '_' + self.clean(str(new_value)) for new_value in value]) + ')'
+                        return (
+                            "("
+                            + "; ".join(
+                                [
+                                    self.clean() + "_" + self.clean(str(new_value))
+                                    for new_value in value
+                                ]
+                            )
+                            + ")"
+                        )
             else:
                 if value_as_term:
                     return self.clean() + '("' + str(value) + '")'
                 else:
-                    return self.clean() + '_' + self.clean(str(value))
+                    return self.clean() + "_" + self.clean(str(value))
         else:
             # It is a Boolean atom
             if self.values[self.boolean_true] == value:
                 return self.clean()
             elif self.values[1 - self.boolean_true] == value:
-                return '\+' + self.clean()
+                return "\+" + self.clean()
             else:
-                raise Exception('Unknown value: {} = {}'.format(self.name, value))
+                raise Exception("Unknown value: {} = {}".format(self.name, value))
 
     def __str__(self):
-        return '{}'.format(self.name)
+        return "{}".format(self.name)
 
 
 class Factor(object):
@@ -535,18 +605,20 @@ class Factor(object):
         elif isinstance(table, dict):
             self.table = table
         else:
-            raise ValueError('Unknown type (expected list, tuple or dict): {}'.format(type(table)))
+            raise ValueError(
+                "Unknown type (expected list, tuple or dict): {}".format(type(table))
+            )
 
     def to_factor(self):
         return self
 
     def copy(self, **kwargs):
         return Factor(
-            rv=kwargs.get('rv', self.rv),
-            pgm=kwargs.get('pgm', self.pgm),
-            name=kwargs.get('name', self.name),
-            parents=kwargs.get('parents', self.parents),
-            table=kwargs.get('table', self.table)
+            rv=kwargs.get("rv", self.rv),
+            pgm=kwargs.get("pgm", self.pgm),
+            name=kwargs.get("name", self.name),
+            parents=kwargs.get("parents", self.parents),
+            table=kwargs.get("table", self.table),
         )
 
     def compress(self, allow_disjunct=False):
@@ -605,7 +677,9 @@ class Factor(object):
                 iv = 0  # Intrinsic Value
                 for bin_value, bin_labels in bins.items():
                     label_cnt = Counter(bin_labels)
-                    ps = [cnt / len(bin_labels) for cnt in label_cnt.values()]  # Pr(x_i)
+                    ps = [
+                        cnt / len(bin_labels) for cnt in label_cnt.values()
+                    ]  # Pr(x_i)
                     h = -sum([p * math.log(p, 2) for p in ps])  # Entropy
                     r = len(bin_labels) / len(node)
                     ig -= r * h
@@ -625,8 +699,9 @@ class Factor(object):
                 for new_path, new_probs in node:
                     new_table[new_path] = new_probs
                 continue
-            if allow_disjunct \
-                    and len(self.pgm.vars[self.parents[igr_idx]].values) > max(2, len(c)):
+            if allow_disjunct and len(
+                self.pgm.vars[self.parents[igr_idx]].values
+            ) > max(2, len(c)):
                 # If disjuncts are allowed, find values that lead to identical subtrees
                 # and merge them
                 branches = defaultdict(set)
@@ -663,87 +738,103 @@ class Factor(object):
 
     def to_huginnet_node(self, include_layout=False):
         rv = self.pgm.vars[self.rv]
-        lines = ["node {} {{".format(rv.clean()),
-                 "  label = \"{}\";".format(self.rv)]
+        lines = ["node {} {{".format(rv.clean()), '  label = "{}";'.format(self.rv)]
         if include_layout:
             lines += ["  position = (100,100);"]
-        lines += ["  states = ({});".format(' '.join(['"{}"'.format(v) for v in sorted(rv.values)])),
-                 "}\n"]
-        return '\n'.join(lines)
+        lines += [
+            "  states = ({});".format(
+                " ".join(['"{}"'.format(v) for v in sorted(rv.values)])
+            ),
+            "}\n",
+        ]
+        return "\n".join(lines)
 
     def to_huginnet_potential(self, include_layout=False):
         rv = self.pgm.vars[self.rv]
         name = rv.clean()
-        value_idxs = sorted((v,i) for i,v in enumerate(rv.values))
+        value_idxs = sorted((v, i) for i, v in enumerate(rv.values))
         if len(self.parents) > 0:
-            name += ' | ' + ' '.join([rv.clean(p) for p in self.parents])
-        lines = ['potential ({}) {{'.format(name),
-                 '  % ' + ' '.join([str(v) for v, _ in value_idxs]),
-                 '  data = (']
+            name += " | " + " ".join([rv.clean(p) for p in self.parents])
+        lines = [
+            "potential ({}) {{".format(name),
+            "  % " + " ".join([str(v) for v, _ in value_idxs]),
+            "  data = (",
+        ]
         table = sorted(self.table.items())
         for k, probs in table:
-            lines.append('    ' + ' '.join([str(probs[value_idx[1]]) for value_idx in value_idxs]) + ' % ' + ' '.join([str(kk) for kk in k]))
-        lines += ['  );',
-                  '}\n']
-        return '\n'.join(lines)
+            lines.append(
+                "    "
+                + " ".join([str(probs[value_idx[1]]) for value_idx in value_idxs])
+                + " % "
+                + " ".join([str(kk) for kk in k])
+            )
+        lines += ["  );", "}\n"]
+        return "\n".join(lines)
 
     def to_xdsl_cpt(self):
         rv = self.pgm.vars[self.rv]
         lines = ['    <cpt id="{}">'.format(rv.clean())]
-        value_idxs = sorted((v,i) for i,v in enumerate(rv.values))
+        value_idxs = sorted((v, i) for i, v in enumerate(rv.values))
         for v, _ in value_idxs:
             lines.append('      <state id="{}" />'.format(v))
         if len(self.parents) > 0:
-            lines.append('      <parents>{}</parents>'.format(' '.join([rv.clean(p) for p in self.parents])))
+            lines.append(
+                "      <parents>{}</parents>".format(
+                    " ".join([rv.clean(p) for p in self.parents])
+                )
+            )
         table = sorted(self.table.items())
-        probs = ' '.join([str(probs[value_idx[1]]) for k, probs in table for value_idx in value_idxs])
-        lines.append('      <probabilities>{}</probabilities>'.format(probs))
-        lines.append('    </cpt>')
-        return '\n'.join(lines)
+        probs = " ".join(
+            [str(probs[value_idx[1]]) for k, probs in table for value_idx in value_idxs]
+        )
+        lines.append("      <probabilities>{}</probabilities>".format(probs))
+        lines.append("    </cpt>")
+        return "\n".join(lines)
 
     def to_xdsl_node(self):
         rv = self.pgm.vars[self.rv]
         lines = [
             '      <node id="{}">'.format(rv.clean()),
-            '        <name>{}</name>'.format(self.rv),
+            "        <name>{}</name>".format(self.rv),
             '        <interior color="e5f6f7" />',
             '        <outline color="000080" />',
             '        <font color="000000" name="Arial" size="8" />',
-            '        <position>100 100 150 150</position>',
-            '      </node>']
-        return '\n'.join(lines)
+            "        <position>100 100 150 150</position>",
+            "      </node>",
+        ]
+        return "\n".join(lines)
 
     def to_xmlbif_cpt(self):
         rv = self.pgm.vars[self.rv]
-        lines = ['<DEFINITION>',
-                 '  <FOR>{}</FOR>'.format(rv.name)]
+        lines = ["<DEFINITION>", "  <FOR>{}</FOR>".format(rv.name)]
         for parent in self.parents:
-            lines.append('  <GIVEN>{}</GIVEN>'.format(parent))
-        value_idxs = sorted((v,i) for i,v in enumerate(rv.values))
+            lines.append("  <GIVEN>{}</GIVEN>".format(parent))
+        value_idxs = sorted((v, i) for i, v in enumerate(rv.values))
         table = sorted(self.table.items())
-        probs = ' '.join([str(probs[value_idx[1]]) for k, probs in table for value_idx in value_idxs])
-        lines.append('  <TABLE>{}</TABLE>'.format(probs))
-        lines.append('</DEFINITION>')
-        return '\n'.join(lines)
+        probs = " ".join(
+            [str(probs[value_idx[1]]) for k, probs in table for value_idx in value_idxs]
+        )
+        lines.append("  <TABLE>{}</TABLE>".format(probs))
+        lines.append("</DEFINITION>")
+        return "\n".join(lines)
 
     def to_xmlbif_node(self):
         rv = self.pgm.vars[self.rv]
-        lines = [
-            '<VARIABLE TYPE="nature">',
-            '  <NAME>{}</NAME>'.format(rv)
-        ]
+        lines = ['<VARIABLE TYPE="nature">', "  <NAME>{}</NAME>".format(rv)]
         for v in sorted(rv.values):
-            lines += ['  <OUTCOME>{}</OUTCOME>'.format(v)]
-        lines += ['</VARIABLE>']
-        return '\n'.join(lines)
+            lines += ["  <OUTCOME>{}</OUTCOME>".format(v)]
+        lines += ["</VARIABLE>"]
+        return "\n".join(lines)
 
     def to_uai08_preamble(self, cpds):
         function_size = 1 + len(self.parents)
         rv_to_idx = {}
         for idx, cpd in enumerate(cpds):
             rv_to_idx[cpd.rv] = idx
-        variables = [str(rv_to_idx[rv]) for rv in self.parents] + [str(rv_to_idx[self.rv])]
-        return '{} {}'.format(function_size, ' '.join(variables))
+        variables = [str(rv_to_idx[rv]) for rv in self.parents] + [
+            str(rv_to_idx[self.rv])
+        ]
+        return "{} {}".format(function_size, " ".join(variables))
 
     def to_uai08_function(self):
         rv = self.pgm.vars[self.rv]
@@ -752,14 +843,21 @@ class Factor(object):
         try:
             table = sorted(self.table.items())
         except TypeError:
-            print('ERROR: Cannot convert compressed CPTs to UAI08 format')
+            print("ERROR: Cannot convert compressed CPTs to UAI08 format")
             sys.exit(1)
         for k, v in table:
-            lines.append(' ' + ' '.join([str(vv) for vv in v]))
-        lines.append('')
-        return '\n'.join(lines)
+            lines.append(" " + " ".join([str(vv) for vv in v]))
+        lines.append("")
+        return "\n".join(lines)
 
-    def to_problog(self, pgm, drop_zero=False, use_neglit=False, value_as_term=True, ad_is_function=False):
+    def to_problog(
+        self,
+        pgm,
+        drop_zero=False,
+        use_neglit=False,
+        value_as_term=True,
+        ad_is_function=False,
+    ):
         lines = []
         var = pgm.vars[self.rv]  # type: Variable
         name = var.clean()
@@ -771,81 +869,127 @@ class Factor(object):
 
         line_cnt = 0
         for k, v in table:
-            if var.boolean_true is not None and drop_zero and v[var.boolean_true] == 0.0 and not use_neglit:
+            if (
+                var.boolean_true is not None
+                and drop_zero
+                and v[var.boolean_true] == 0.0
+                and not use_neglit
+            ):
                 continue
             head_problits = []
             if var.boolean_true is None:
                 for idx, vv in enumerate(v):
                     if not (drop_zero and vv == 0.0):
-                        head_problits.append((vv, var.to_problog_value(var.values[idx], value_as_term)))
+                        head_problits.append(
+                            (vv, var.to_problog_value(var.values[idx], value_as_term))
+                        )
             elif isinstance(var.boolean_true, int):
                 if drop_zero and v[var.boolean_true] == 0.0 and use_neglit:
-                    head_problits.append((None, var.to_problog_value(var.values[1 - var.boolean_true], value_as_term)))
+                    head_problits.append(
+                        (
+                            None,
+                            var.to_problog_value(
+                                var.values[1 - var.boolean_true], value_as_term
+                            ),
+                        )
+                    )
                 elif v[var.boolean_true] == 1.0:
-                    head_problits.append((None, var.to_problog_value(var.values[var.boolean_true], value_as_term)))
+                    head_problits.append(
+                        (
+                            None,
+                            var.to_problog_value(
+                                var.values[var.boolean_true], value_as_term
+                            ),
+                        )
+                    )
                 else:
-                    head_problits.append((v[var.boolean_true],
-                                          var.to_problog_value(var.values[var.boolean_true], value_as_term)))
+                    head_problits.append(
+                        (
+                            v[var.boolean_true],
+                            var.to_problog_value(
+                                var.values[var.boolean_true], value_as_term
+                            ),
+                        )
+                    )
             body_lits = []
             for parent, parent_value in zip(self.parents, k):
                 if parent_value is not None:
                     parent_var = pgm.vars[parent]
-                    body_lits.append(parent_var.to_problog_value(parent_value, value_as_term))
+                    body_lits.append(
+                        parent_var.to_problog_value(parent_value, value_as_term)
+                    )
 
             if ad_is_function:
                 for head_cnt, (head_prob, head_lit) in enumerate(head_problits):
                     if len(body_lits) == 0:
-                        lines.append('({};1.0)::{}.'.format(head_prob, head_lit))
+                        lines.append("({};1.0)::{}.".format(head_prob, head_lit))
                     else:
                         # new_probfact = 'pf_{}_{}_{}'.format(self.rv_clean(), line_cnt, head_cnt)
                         # new_body_lits = body_lits + [new_probfact]
                         # lines.append('({};1.0)::{}.'.format(head_prob, new_probfact))
                         # lines.append('{} :- {}.'.format(head_lit, ', '.join(new_body_lits)))
-                        lines.append('({};1.0)::{} :- {}.'.format(head_prob, head_lit, ', '.join(body_lits)))
+                        lines.append(
+                            "({};1.0)::{} :- {}.".format(
+                                head_prob, head_lit, ", ".join(body_lits)
+                            )
+                        )
             else:
                 if len(body_lits) > 0:
-                    body_str = ' :- ' + ', '.join(body_lits)
+                    body_str = " :- " + ", ".join(body_lits)
                 else:
-                    body_str = ''
+                    body_str = ""
                 head_strs = []
                 for prob, lit in head_problits:
                     if prob is None:
                         head_strs.append(str(lit))
                     else:
-                        head_strs.append('{}::{}'.format(prob, lit))
-                head_str = '; '.join(head_strs)
-                lines.append('{}{}.'.format(head_str, body_str))
+                        head_strs.append("{}::{}".format(prob, lit))
+                head_str = "; ".join(head_strs)
+                lines.append("{}{}.".format(head_str, body_str))
             line_cnt += 1
 
         if ad_is_function and var.boolean_true is None:
-            head_lits = [var.to_problog_value(value, value_as_term) for value in var.values]
+            head_lits = [
+                var.to_problog_value(value, value_as_term) for value in var.values
+            ]
             # lines.append('false_constraints :- '+', '.join(['\+'+l for l in head_lits])+'.')
-            lines.append('constraint([' + ', '.join(['\+' + l for l in head_lits]) + '], false).')
+            lines.append(
+                "constraint([" + ", ".join(["\+" + l for l in head_lits]) + "], false)."
+            )
             for lit1, lit2 in itertools.combinations(head_lits, 2):
                 # lines.append('false_constraints :- {}, {}.'.format(lit1, lit2))
-                lines.append('constraint([{}, {}], false).'.format(lit1, lit2))
+                lines.append("constraint([{}, {}], false).".format(lit1, lit2))
 
-        return '\n'.join(lines)
+        return "\n".join(lines)
 
-    def to_problog_undirected(self, pgm, drop_zero=False, use_neglit=False, value_as_term=True, ad_is_function=False):
+    def to_problog_undirected(
+        self,
+        pgm,
+        drop_zero=False,
+        use_neglit=False,
+        value_as_term=True,
+        ad_is_function=False,
+    ):
         lines = []
         name = self.name
         table = self.table.items()
         line_cnt = 0
         for k, v in table:
-            assert (len(v) == 1)
+            assert len(v) == 1
             body_lits = []
             for parent, parent_value in zip(self.parents, k):
                 if parent_value is not None:
                     parent_var = pgm.vars[parent]
-                    body_lits.append(parent_var.to_problog_value(parent_value, value_as_term))
+                    body_lits.append(
+                        parent_var.to_problog_value(parent_value, value_as_term)
+                    )
 
-            body_str = ' :- ' + ', '.join(body_lits)
-            head_str = '{}::{}({})'.format(v[0], name, line_cnt)
-            lines.append('{}{}.'.format(head_str, body_str))
+            body_str = " :- " + ", ".join(body_lits)
+            head_str = "{}::{}({})".format(v[0], name, line_cnt)
+            lines.append("{}{}.".format(head_str, body_str))
             line_cnt += 1
 
-        return '\n'.join(lines)
+        return "\n".join(lines)
 
     def __str__(self):
         lines = []
@@ -854,19 +998,21 @@ class Factor(object):
         except TypeError:
             table = self.table.items()
         for k, v in table:
-            lines.append('{}: {}'.format(k, v))
-        table = '\n'.join(lines)
-        parents = ''
+            lines.append("{}: {}".format(k, v))
+        table = "\n".join(lines)
+        parents = ""
         if len(self.parents) > 0:
-            parents = ', '.join(map(str, self.parents))
+            parents = ", ".join(map(str, self.parents))
 
         if self.rv is not None:
             var = self.pgm.vars[self.rv]
             rv = var.name
             if len(self.parents) > 0:
-                rv += ' | '
-            return 'Factor ({}{}) = {}\n{}\n'.format(rv, parents, ', '.join([str(v) for v in var.values]), table)
-        return 'Factor ({})\n{}\n'.format(parents, table)
+                rv += " | "
+            return "Factor ({}{}) = {}\n{}\n".format(
+                rv, parents, ", ".join([str(v) for v in var.values]), table
+            )
+        return "Factor ({})\n{}\n".format(parents, table)
 
 
 class OrCPT(Factor):
@@ -906,8 +1052,10 @@ class OrCPT(Factor):
 
     def __str__(self):
         rv = self.pgm.vars[self.rv]
-        table = '\n'.join(['{}'.format(pv) for pv in self.parentvalues])
-        parents = ''
+        table = "\n".join(["{}".format(pv) for pv in self.parentvalues])
+        parents = ""
         if len(self.parents) > 0:
-            parents = ' -- {}'.format(','.join(self.parents))
-        return 'OrCPT {} [{}]{}\n{}\n'.format(self.rv, ','.join(map(str, rv.values)), parents, table)
+            parents = " -- {}".format(",".join(self.parents))
+        return "OrCPT {} [{}]{}\n{}\n".format(
+            self.rv, ",".join(map(str, rv.values)), parents, table
+        )
