@@ -19,28 +19,28 @@ limitations under the License.
 import os
 import sys
 
-sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
+sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
 
 from problog.util import load_module
 from problog import version
 
 problog_tasks = {}
-problog_tasks['prob'] = 'problog.tasks.probability'
-problog_tasks['mpe'] = 'problog.tasks.mpe'
-problog_tasks['sample'] = 'problog.tasks.sample'
-problog_tasks['ground'] = 'problog.tasks.ground'
-problog_tasks['lfi'] = 'problog.learning.lfi'
-problog_tasks['explain'] = 'problog.tasks.explain'
-problog_tasks['web'] = 'problog.web.server'
-problog_tasks['dt'] = 'problog.tasks.dtproblog'
-problog_tasks['shell'] = 'problog.tasks.shell'
-problog_tasks['bn'] = 'problog.tasks.bayesnet'
-problog_tasks['parse'] = 'problog.parser'
-problog_tasks['map'] = 'problog.tasks.map'
-problog_tasks['time'] = 'problog.tasks.time1'
-problog_tasks['constraint'] = 'problog.tasks.constraint'
+problog_tasks["prob"] = "problog.tasks.probability"
+problog_tasks["mpe"] = "problog.tasks.mpe"
+problog_tasks["sample"] = "problog.tasks.sample"
+problog_tasks["ground"] = "problog.tasks.ground"
+problog_tasks["lfi"] = "problog.learning.lfi"
+problog_tasks["explain"] = "problog.tasks.explain"
+problog_tasks["web"] = "problog.web.server"
+problog_tasks["dt"] = "problog.tasks.dtproblog"
+problog_tasks["shell"] = "problog.tasks.shell"
+problog_tasks["bn"] = "problog.tasks.bayesnet"
+problog_tasks["parse"] = "problog.parser"
+problog_tasks["map"] = "problog.tasks.map"
+problog_tasks["time"] = "problog.tasks.time1"
+problog_tasks["constraint"] = "problog.tasks.constraint"
 
-problog_default_task = 'prob'
+problog_default_task = "prob"
 
 
 def run_task(argv):
@@ -79,21 +79,26 @@ def main(argv=None):
         argv = sys.argv[1:]
 
     if len(argv) > 0:
-        if argv[0] == 'install':
+        if argv[0] == "install":
             from .. import setup
+
             setup.install()
             return
-        elif argv[0] == 'info':
+        elif argv[0] == "info":
             from problog.core import list_transformations
+
             list_transformations()
             return
-        elif argv[0] == 'unittest':
+        elif argv[0] == "unittest":
             import unittest
+
             test_results = unittest.TextTestResult(sys.stderr, False, 1)
-            unittest.TestLoader().discover(os.path.join(os.path.dirname(__file__), '..')).run(test_results)
+            unittest.TestLoader().discover(
+                os.path.join(os.path.dirname(__file__), "..")
+            ).run(test_results)
             return
-        elif argv[0] == '--version':
-            print (version.version)
+        elif argv[0] == "--version":
+            print(version.version)
             return
         else:
             return run_task(argv)
@@ -101,5 +106,5 @@ def main(argv=None):
         return run_task(argv)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main(sys.argv[1:])
